@@ -18,31 +18,29 @@ Login::Login(QObject *parent)
 void Login::init()
 {
     m_homeserverReachable = false;
-    m_connection = nullptr;
+    m_connection = new Connection();
     m_matrixId = QString();
     m_password = QString();
-    m_deviceName = QString();
+    m_deviceName = QStringLiteral("NeoChat %1 %2 %3 %4")
+                       .arg(QSysInfo::machineHostName(), QSysInfo::productType(), QSysInfo::productVersion(), QSysInfo::currentCpuArchitecture());
     m_supportsSso = false;
     m_supportsPassword = false;
     m_ssoUrl = QUrl();
 
-    connect(this, &Login::matrixIdChanged, this, [=]() {
+    connect(this, &Login::matrixIdChanged, this, [this]() {
         setHomeserverReachable(false);
 
-        if (m_connection) {
-            delete m_connection;
-            m_connection = nullptr;
-        }
-
         if (m_matrixId == "@") {
             return;
         }
 
         m_testing = true;
         Q_EMIT testingChanged();
-        m_connection = new Connection(this);
+        if (!m_connection) {
+            m_connection = new Connection();
+        }
         m_connection->resolveServer(m_matrixId);
-        connect(m_connection, &Connection::loginFlowsChanged, this, [=]() {
+        connectSingleShot(m_connection, &Connection::loginFlowsChanged, this, [this]() {
             setHomeserverReachable(true);
             m_testing = false;
             Q_EMIT testingChanged();
@@ -51,6 +49,42 @@ void Login::init()
             Q_EMIT loginFlowsChanged();
         });
     });
+    connect(m_connection, &Connection::connected, this, [this] {
+        Q_EMIT connected();
+        m_isLoggingIn = false;
+        Q_EMIT isLoggingInChanged();
+        Q_ASSERT(m_connection);
+        AccountSettings account(m_connection->userId());
+        account.setKeepLoggedIn(true);
+        account.setHomeserver(m_connection->homeserver());
+        account.setDeviceId(m_connection->deviceId());
+        account.setDeviceName(m_deviceName);
+        if (!Controller::instance().saveAccessTokenToKeyChain(account, m_connection->accessToken())) {
+            qWarning() << "Couldn't save access token";
+        }
+        account.sync();
+        Controller::instance().addConnection(m_connection);
+        Controller::instance().setActiveConnection(m_connection);
+        m_connection = nullptr;
+    });
+    connect(m_connection, &Connection::networkError, this, [this](QString error, const QString &, int, int) {
+        Q_EMIT Controller::instance().globalErrorOccured(i18n("Network Error"), std::move(error));
+        m_isLoggingIn = false;
+        Q_EMIT isLoggingInChanged();
+    });
+    connect(m_connection, &Connection::loginError, this, [this](QString error, const QString &) {
+        Q_EMIT errorOccured(i18n("Login Failed: %1", error));
+        m_isLoggingIn = false;
+        Q_EMIT isLoggingInChanged();
+    });
+
+    connect(m_connection, &Connection::resolveError, this, [](QString error) {
+        Q_EMIT Controller::instance().globalErrorOccured(i18n("Network Error"), std::move(error));
+    });
+
+    connectSingleShot(m_connection, &Connection::syncDone, this, [this]() {
+        Q_EMIT Controller::instance().initiated();
+    });
 }
 
 void Login::setHomeserverReachable(bool reachable)
@@ -105,51 +139,7 @@ void Login::login()
     m_isLoggingIn = true;
     Q_EMIT isLoggingInChanged();
 
-    setDeviceName("NeoChat " + QSysInfo::machineHostName() + " " + QSysInfo::productType() + " " + QSysInfo::productVersion() + " "
-                  + QSysInfo::currentCpuArchitecture());
-
-    m_connection = new Connection(this);
-    m_connection->resolveServer(m_matrixId);
-
-    connect(m_connection, &Connection::loginFlowsChanged, this, [=]() {
-        m_connection->loginWithPassword(m_matrixId, m_password, m_deviceName, QString());
-        connect(m_connection, &Connection::connected, this, [=] {
-            Q_EMIT connected();
-            m_isLoggingIn = false;
-            Q_EMIT isLoggingInChanged();
-            AccountSettings account(m_connection->userId());
-            account.setKeepLoggedIn(true);
-            account.clearAccessToken(); // Drop the legacy - just in case
-            account.setHomeserver(m_connection->homeserver());
-            account.setDeviceId(m_connection->deviceId());
-            account.setDeviceName(m_deviceName);
-            if (!Controller::instance().saveAccessTokenToKeyChain(account, m_connection->accessToken())) {
-                qWarning() << "Couldn't save access token";
-            }
-            account.sync();
-            Controller::instance().addConnection(m_connection);
-            Controller::instance().setActiveConnection(m_connection);
-        });
-        connect(m_connection, &Connection::networkError, [=](QString error, const QString &, int, int) {
-            Q_EMIT Controller::instance().globalErrorOccured(i18n("Network Error"), std::move(error));
-            m_isLoggingIn = false;
-            Q_EMIT isLoggingInChanged();
-        });
-        connect(m_connection, &Connection::loginError, [=](QString error, const QString &) {
-            Q_EMIT errorOccured(i18n("Login Failed: %1", error));
-            m_isLoggingIn = false;
-            Q_EMIT isLoggingInChanged();
-        });
-    });
-
-    connect(m_connection, &Connection::resolveError, this, [=](QString error) {
-        Q_EMIT Controller::instance().globalErrorOccured(i18n("Network Error"), std::move(error));
-    });
-
-    connect(m_connection, &Connection::syncDone, this, [=]() {
-        Q_EMIT initialSyncFinished();
-        disconnect(m_connection, &Connection::syncDone, this, nullptr);
-    });
+    m_connection->loginWithPassword(m_matrixId, m_password, m_deviceName, QString());
 }
 
 bool Login::supportsPassword() const
@@ -169,29 +159,12 @@ QUrl Login::ssoUrl() const
 
 void Login::loginWithSso()
 {
-    SsoSession *session = m_connection->prepareForSso("NeoChat " + QSysInfo::machineHostName() + " " + QSysInfo::productType() + " "
-                                                      + QSysInfo::productVersion() + " " + QSysInfo::currentCpuArchitecture());
-    m_ssoUrl = session->ssoUrl();
+    m_connection->resolveServer(m_matrixId);
+    connectSingleShot(m_connection, &Connection::loginFlowsChanged, this, [this]() {
+        SsoSession *session = m_connection->prepareForSso(m_deviceName);
+        m_ssoUrl = session->ssoUrl();
+    });
     Q_EMIT ssoUrlChanged();
-    connect(m_connection, &Connection::connected, [=]() {
-        Q_EMIT connected();
-        AccountSettings account(m_connection->userId());
-        account.setKeepLoggedIn(true);
-        account.clearAccessToken(); // Drop the legacy - just in case
-        account.setHomeserver(m_connection->homeserver());
-        account.setDeviceId(m_connection->deviceId());
-        account.setDeviceName(m_deviceName);
-        if (!Controller::instance().saveAccessTokenToKeyChain(account, m_connection->accessToken())) {
-            qWarning() << "Couldn't save access token";
-        }
-        account.sync();
-        Controller::instance().addConnection(m_connection);
-        Controller::instance().setActiveConnection(m_connection);
-    });
-    connect(m_connection, &Connection::syncDone, this, [=]() {
-        Q_EMIT initialSyncFinished();
-        disconnect(m_connection, &Connection::syncDone, this, nullptr);
-    });
 }
 
 bool Login::testing() const
