@@ -6,10 +6,12 @@
 #include <QFontDatabase>
 #include <QGuiApplication>
 #include <QIcon>
+#include <QNetworkAccessManager>
 #include <QNetworkProxy>
 #include <QNetworkProxyFactory>
 #include <QQmlApplicationEngine>
 #include <QQmlContext>
+#include <QQmlNetworkAccessManagerFactory>
 #include <QQuickStyle>
 #include <QQuickWindow>
 
@@ -44,6 +46,7 @@
 #include "chatboxhelper.h"
 #include "chatdocumenthandler.h"
 #include "clipboard.h"
+#include "collapsestateproxymodel.h"
 #include "commandmodel.h"
 #include "controller.h"
 #include "csapi/joining.h"
@@ -60,6 +63,7 @@
 #include "neochatconfig.h"
 #include "neochatroom.h"
 #include "neochatuser.h"
+#include "networkaccessmanager.h"
 #include "notificationsmanager.h"
 #include "publicroomlistmodel.h"
 #include "roomlistmodel.h"
@@ -74,8 +78,21 @@
 #include "colorschemer.h"
 #endif
 
+#ifdef HAVE_RUNNER
+#include "runner.h"
+#include <QDBusConnection>
+#endif
+
 using namespace Quotient;
 
+class NetworkAccessManagerFactory : public QQmlNetworkAccessManagerFactory
+{
+    QNetworkAccessManager *create(QObject *) override
+    {
+        return NetworkAccessManager::instance();
+    }
+};
+
 #ifdef HAVE_WINDOWSYSTEM
 static void raiseWindow(QWindow *window)
 {
@@ -187,6 +204,7 @@ int main(int argc, char *argv[])
     qmlRegisterType<UserListModel>("org.kde.neochat", 1, 0, "UserListModel");
     qmlRegisterType<CustomEmojiModel>("org.kde.neochat", 1, 0, "CustomEmojiModel");
     qmlRegisterType<MessageEventModel>("org.kde.neochat", 1, 0, "MessageEventModel");
+    qmlRegisterType<CollapseStateProxyModel>("org.kde.neochat", 1, 0, "CollapseStateProxyModel");
     qmlRegisterType<MessageFilterModel>("org.kde.neochat", 1, 0, "MessageFilterModel");
     qmlRegisterType<PublicRoomListModel>("org.kde.neochat", 1, 0, "PublicRoomListModel");
     qmlRegisterType<UserDirectoryListModel>("org.kde.neochat", 1, 0, "UserDirectoryListModel");
@@ -219,6 +237,7 @@ int main(int argc, char *argv[])
     engine.rootContext()->setContextObject(new KLocalizedContext(&engine));
     KLocalizedString::setApplicationDomain("neochat");
     QObject::connect(&engine, &QQmlApplicationEngine::quit, &app, &QCoreApplication::quit);
+    engine.setNetworkAccessManagerFactory(new NetworkAccessManagerFactory());
 
     QCommandLineParser parser;
     parser.setApplicationDescription(i18n("Client for the matrix communication protocol"));
@@ -243,6 +262,11 @@ int main(int argc, char *argv[])
         RoomManager::instance().setUrlArgument(parser.positionalArguments()[0]);
     }
 
+#ifdef HAVE_RUNNER
+    Runner runner;
+    QDBusConnection::sessionBus().registerObject("/RoomRunner", &runner, QDBusConnection::ExportScriptableContents);
+#endif
+
 #ifdef HAVE_KDBUSADDONS
     KDBusService service(KDBusService::Unique);
     service.connect(&service,
