@@ -5,6 +5,7 @@
 
 #include "neochatconfig.h"
 #include <connection.h>
+#include <csapi/rooms.h>
 #include <events/reactionevent.h>
 #include <events/redactionevent.h>
 #include <events/roomavatarevent.h>
@@ -15,6 +16,7 @@
 #include "stickerevent.h"
 
 #include <QDebug>
+#include <QGuiApplication>
 #include <QQmlEngine> // for qmlRegisterType()
 #include <QTimeZone>
 
@@ -40,7 +42,9 @@ QHash<int, QByteArray> MessageEventModel::roleNames() const
     roles[FileMimetypeIcon] = "fileMimetypeIcon";
     roles[AnnotationRole] = "annotation";
     roles[EventResolvedTypeRole] = "eventResolvedType";
+    roles[IsReplyRole] = "isReply";
     roles[ReplyRole] = "reply";
+    roles[ReplyIdRole] = "replyId";
     roles[UserMarkerRole] = "userMarker";
     roles[ShowAuthorRole] = "showAuthor";
     roles[ShowSectionRole] = "showSection";
@@ -49,6 +53,8 @@ QHash<int, QByteArray> MessageEventModel::roleNames() const
     roles[SourceRole] = "source";
     roles[MimeTypeRole] = "mimeType";
     roles[FormattedBodyRole] = "formattedBody";
+    roles[AuthorIdRole] = "authorId";
+    roles[MediaUrlRole] = "mediaUrl";
     return roles;
 }
 
@@ -60,20 +66,8 @@ MessageEventModel::MessageEventModel(QObject *parent)
     qmlRegisterAnonymousType<FileTransferInfo>("org.kde.neochat", 1);
     qRegisterMetaType<FileTransferInfo>();
 
-    QTimer::singleShot(0, this, [this]() {
-        if (!m_currentRoom) {
-            return;
-        }
-        m_currentRoom->getPreviousContent(50);
-        connect(this, &QAbstractListModel::rowsInserted, this, [this]() {
-            if (m_currentRoom->readMarkerEventId().isEmpty()) {
-                return;
-            }
-            const auto it = m_currentRoom->findInTimeline(m_currentRoom->readMarkerEventId());
-            if (it == m_currentRoom->historyEdge()) {
-                m_currentRoom->getPreviousContent(50);
-            }
-        });
+    connect(static_cast<QGuiApplication *>(QGuiApplication::instance()), &QGuiApplication::paletteChanged, this, [this] {
+        Q_EMIT dataChanged(index(0, 0), index(rowCount() - 1, 0), {AuthorRole, ReplyRole});
     });
 }
 
@@ -94,7 +88,7 @@ void MessageEventModel::setRoom(NeoChatRoom *room)
     if (room) {
         m_lastReadEventIndex = QPersistentModelIndex(QModelIndex());
         room->setDisplayed();
-        if (m_currentRoom->timelineSize() < 10) {
+        if (m_currentRoom->timelineSize() < 10 && !room->allHistoryLoaded()) {
             room->getPreviousContent(50);
         }
         lastReadEventId = room->readMarkerEventId();
@@ -638,19 +632,35 @@ QVariant MessageEventModel::data(const QModelIndex &idx, int role) const
         return variantList;
     }
 
+    if (role == IsReplyRole) {
+        return !evt.contentJson()["m.relates_to"].toObject()["m.in_reply_to"].toObject()["event_id"].toString().isEmpty();
+    }
+
+    if (role == ReplyIdRole) {
+        return evt.contentJson()["m.relates_to"].toObject()["m.in_reply_to"].toObject()["event_id"].toString();
+    }
+
     if (role == ReplyRole) {
         const QString &replyEventId = evt.contentJson()["m.relates_to"].toObject()["m.in_reply_to"].toObject()["event_id"].toString();
         if (replyEventId.isEmpty()) {
             return {};
         };
         const auto replyIt = m_currentRoom->findInTimeline(replyEventId);
-        if (replyIt == m_currentRoom->historyEdge()) {
+        const RoomEvent *replyPtr = replyIt != m_currentRoom->historyEdge() ? &**replyIt : nullptr;
+        if (!replyPtr) {
+            for (const auto &e : m_extraEvents) {
+                if (e->id() == replyEventId) {
+                    replyPtr = e.get();
+                    break;
+                }
+            }
+        }
+        if (!replyPtr) {
             return {};
-        };
-        const auto &replyEvt = **replyIt;
+        }
 
         QString type;
-        if (auto e = eventCast<const RoomMessageEvent>(&replyEvt)) {
+        if (auto e = eventCast<const RoomMessageEvent>(replyPtr)) {
             switch (e->msgtype()) {
             case MessageEventType::Emote:
                 type = "emote";
@@ -675,29 +685,29 @@ QVariant MessageEventModel::data(const QModelIndex &idx, int role) const
                 type = "message";
             }
 
-        } else if (is<const StickerEvent>(replyEvt)) {
+        } else if (is<const StickerEvent>(*replyPtr)) {
             type = "sticker";
         } else {
             type = "other";
         }
 
         QVariant content;
-        if (auto e = eventCast<const RoomMessageEvent>(&replyEvt)) {
+        if (auto e = eventCast<const RoomMessageEvent>(replyPtr)) {
             // Cannot use e.contentJson() here because some
             // EventContent classes inject values into the copy of the
             // content JSON stored in EventContent::Base
             content = e->hasFileContent() ? QVariant::fromValue(e->content()->originalJson) : QVariant();
         };
 
-        if (auto e = eventCast<const StickerEvent>(&replyEvt)) {
+        if (auto e = eventCast<const StickerEvent>(replyPtr)) {
             content = QVariant::fromValue(e->image().originalJson);
         }
 
         return QVariantMap{{"eventId", replyEventId},
-                           {"display", m_currentRoom->eventToString(replyEvt, Qt::RichText)},
+                           {"display", m_currentRoom->eventToString(*replyPtr, Qt::RichText)},
                            {"content", content},
                            {"type", type},
-                           {"author", userAtEvent(static_cast<NeoChatUser *>(m_currentRoom->user(replyEvt.senderId())), m_currentRoom, evt)}};
+                           {"author", userAtEvent(static_cast<NeoChatUser *>(m_currentRoom->user(replyPtr->senderId())), m_currentRoom, evt)}};
     }
 
     if (role == ShowAuthorRole) {
@@ -759,6 +769,26 @@ QVariant MessageEventModel::data(const QModelIndex &idx, int role) const
 
         return res;
     }
+    if (role == AuthorIdRole) {
+        return evt.senderId();
+    }
+
+    if (role == MediaUrlRole) {
+#ifdef QUOTIENT_07
+        if (auto e = eventCast<const RoomMessageEvent>(&evt)) {
+            if (!e->hasFileContent()) {
+                return QVariant();
+            }
+            if (e->content()->originalJson.contains(QStringLiteral("file")) && e->content()->originalJson["file"].toObject().contains(QStringLiteral("url"))) {
+                return m_currentRoom->makeMediaUrl(e->id(), e->content()->originalJson["file"]["url"].toString());
+            }
+            if (e->content()->originalJson.contains(QStringLiteral("url"))) {
+                return m_currentRoom->makeMediaUrl(e->id(), e->content()->originalJson["url"].toString());
+            }
+        }
+#endif
+        return m_currentRoom->urlToDownload(evt.id());
+    }
 
     return {};
 }
@@ -847,3 +877,13 @@ QVariant MessageEventModel::getLatestMessageFromIndex(const int baseline)
     }
     return replyResponse;
 }
+
+void MessageEventModel::loadReply(const QModelIndex &index)
+{
+    auto job = m_currentRoom->connection()->callApi<GetOneRoomEventJob>(m_currentRoom->id(), data(index, ReplyIdRole).toString());
+    QPersistentModelIndex persistentIndex(index);
+    connect(job, &BaseJob::success, this, [this, job, persistentIndex] {
+        m_extraEvents.push_back(fromJson<event_ptr_tt<RoomEvent>>(job->jsonData()));
+        Q_EMIT dataChanged(persistentIndex, persistentIndex, {ReplyRole});
+    });
+}
