@@ -252,6 +252,35 @@ QDateTime NeoChatRoom::lastActiveTime()
     return messageEvents().rbegin()->get()->originTimestamp();
 }
 
+QString NeoChatRoom::subtitleText()
+{
+    QString subtitle = this->lastEventToString().size() == 0 ? this->topic() : this->lastEventToString();
+
+    subtitle
+        // replace blockquote, i.e. '> text'
+        .replace(QRegularExpression("(\r\n\t|\n|\r\t|)> "), " ")
+        // replace headings, i.e. "# text"
+        .replace(QRegularExpression("(\r\n\t|\n|\r\t|)\\#{1,6} "), " ")
+        // replace newlines
+        .replace(QRegularExpression("(\r\n\t|\n|\r\t)"), " ")
+        // replace '**text**' and '__text__'
+        .replace(QRegularExpression("(\\*\\*|__)(?=\\S)([^\\r]*\\S)\\1"), "\\2")
+        // replace '*text*' and '_text_'
+        .replace(QRegularExpression("(\\*|_)(?=\\S)([^\\r]*\\S)\\1"), "\\2")
+        // replace '~~text~~'
+        .replace(QRegularExpression("~~(.*)~~"), "\\1")
+        // replace '~text~'
+        .replace(QRegularExpression("~(.*)~"), "\\1")
+        // replace '<del>text</del>'
+        .replace(QRegularExpression("<del>(.*)</del>"), "\\1")
+        // replace '```code```'
+        .replace(QRegularExpression("```([^```]+)```"), "\\1")
+        // replace '`code`'
+        .replace(QRegularExpression("`([^`]+)`"), "\\1");
+
+    return subtitle.size() > 0 ? subtitle : QStringLiteral(" ");
+}
+
 int NeoChatRoom::savedTopVisibleIndex() const
 {
     return firstDisplayedMarker() == historyEdge() ? 0 : int(firstDisplayedMarker() - messageEvents().rbegin());
@@ -311,7 +340,11 @@ QVariantMap NeoChatRoom::getUser(const QString &userID) const
 
 QUrl NeoChatRoom::urlToMxcUrl(const QUrl &mxcUrl)
 {
+#ifdef QUOTIENT_07
+    return connection()->makeMediaUrl(mxcUrl);
+#else
     return DownloadFileJob::makeRequestUrl(connection()->homeserver(), mxcUrl);
+#endif
 }
 
 QString NeoChatRoom::avatarMediaId() const
@@ -396,6 +429,18 @@ QString NeoChatRoom::eventToString(const RoomEvent &evt, Qt::TextFormat format,
         [this](const RoomMemberEvent &e) {
             // FIXME: Rewind to the name that was at the time of this event
             auto subjectName = this->htmlSafeMemberName(e.userId());
+            if (e.membership() == MembershipType::Leave) {
+                auto displayName = e.prevContent()->displayName;
+#ifdef QUOTIENT_07
+                if (displayName) {
+                    subjectName = sanitized(*displayName).toHtmlEscaped();
+#else
+                if (displayName.isEmpty()) {
+                    subjectName = sanitized(displayName).toHtmlEscaped();
+#endif
+                }
+            }
+
             // The below code assumes senderName output in AuthorRole
             switch (e.membership()) {
             case MembershipType::Invite:
@@ -493,6 +538,15 @@ QString NeoChatRoom::eventToString(const RoomEvent &evt, Qt::TextFormat format,
             if (e.matrixType() == QLatin1String("m.room.server_acl")) {
                 return i18n("changed the server access control lists for this room");
             }
+            if (e.matrixType() == QLatin1String("im.vector.modular.widgets")) {
+                if (e.fullJson()["unsigned"]["prev_content"].toObject().isEmpty()) {
+                    return i18nc("[User] added <name> widget", "added %1 widget", e.contentJson()["name"].toString());
+                }
+                if (e.contentJson().isEmpty()) {
+                    return i18nc("[User] removed <name> widget", "removed %1 widget", e.fullJson()["unsigned"]["prev_content"]["name"].toString());
+                }
+                return i18nc("[User] configured <name> widget", "configured %1 widget", e.contentJson()["name"].toString());
+            }
             return e.stateKey().isEmpty() ? i18n("updated %1 state", e.matrixType())
                                           : i18n("updated %1 state for %2", e.matrixType(), e.stateKey().toHtmlEscaped());
         },
@@ -781,3 +835,8 @@ QCoro::Task<void> NeoChatRoom::doDeleteMessagesByUser(const QString &user)
         }
     }
 }
+
+void NeoChatRoom::clearInvitationNotification()
+{
+    NotificationsManager::instance().clearInvitationNotification(id());
+}
