@@ -171,7 +171,7 @@ void RoomListModel::connectRoomSignals(NeoChatRoom *room)
         refresh(room);
     });
     connect(room, &Room::addedMessages, this, [this, room] {
-        refresh(room, {LastEventRole});
+        refresh(room, {LastEventRole, SubtitleTextRole});
     });
     connect(room, &Room::notificationCountChanged, this, &RoomListModel::handleNotifications);
     connect(room, &Room::highlightCountChanged, this, [this, room] {
@@ -236,11 +236,11 @@ void RoomListModel::handleNotifications()
                     avatar_image = room->avatar(128);
                 }
                 NotificationsManager::instance().postNotification(dynamic_cast<NeoChatRoom *>(room),
-                                                                  room->displayName(),
                                                                   sender->displayname(room),
                                                                   notification["event"].toObject()["content"].toObject()["body"].toString(),
                                                                   avatar_image,
-                                                                  notification["event"].toObject()["event_id"].toString());
+                                                                  notification["event"].toObject()["event_id"].toString(),
+                                                                  true);
             }
         }
     });
@@ -329,7 +329,7 @@ QVariant RoomListModel::data(const QModelIndex &index, int role) const
     }
     NeoChatRoom *room = m_rooms.at(index.row());
     if (role == NameRole) {
-        return !room->name().isEmpty() ? room->htmlSafeName() : room->htmlSafeDisplayName();
+        return !room->name().isEmpty() ? room->name() : room->displayName();
     }
     if (role == DisplayNameRole) {
         return room->displayName();
@@ -396,6 +396,15 @@ QVariant RoomListModel::data(const QModelIndex &index, int role) const
     if (role == CategoryVisibleRole) {
         return m_categoryVisibility.value(data(index, CategoryRole).toInt(), true);
     }
+    if (role == SubtitleTextRole) {
+        return room->subtitleText();
+    }
+    if (role == AvatarImageRole) {
+        return room->avatar(128);
+    }
+    if (role == IdRole) {
+        return room->id();
+    }
     return QVariant();
 }
 
@@ -426,6 +435,7 @@ QHash<int, QByteArray> RoomListModel::roleNames() const
     roles[JoinStateRole] = "joinState";
     roles[CurrentRoomRole] = "currentRoom";
     roles[CategoryVisibleRole] = "categoryVisible";
+    roles[SubtitleTextRole] = "subtitleText";
     return roles;
 }
 
